@@ -1,4 +1,6 @@
 #!/usr/bin/env python
+#from __future__ import unicode_literals
+#from __future__ import print_function
 ###############################################################################
 # Name: Editra.py                                                             #
 # Purpose: Implements Editras App object and the Main method                  #
@@ -16,8 +18,8 @@ running Editra.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: Editra.py 69061 2011-09-11 17:04:41Z CJP $"
-__revision__ = "$Revision: 69061 $"
+__svnid__ = "$Id: Editra.py 69449 2011-10-18 16:43:34Z CJP $"
+__revision__ = "$Revision: 69449 $"
 
 #--------------------------------------------------------------------------#
 # Dependencies
@@ -29,7 +31,7 @@ import sys
 # So ensure correct version of wxPython can be loaded
 if not hasattr(sys, 'frozen') and 'wx' not in sys.modules:
     import wxversion
-    wxversion.ensureMinimal('2.8')
+    wxversion.ensureMinimal(b'2.8')
 
 import codecs
 import base64
@@ -819,7 +821,7 @@ def InitConfig():
                 profiler.Profile_Set('LAST_SESSION', smgr.DefaultSession)
             else:
                 # After 0.6.58 session is reduced to a name instead of path
-                if os.path.sep in sess:
+                if sess and os.path.sep in sess:
                     name = smgr.SessionNameFromPath(sess)
                     profiler.Profile_Set('LAST_SESSION', name)
 
@@ -1075,8 +1077,8 @@ def _Main(opts, args):
     # Print ipc server authentication info
     if '--auth' in opts:
         opts.pop('--auth')
-        print "port=%d,key=%s" % (ed_ipc.EDPORT,
-                                  profiler.Profile_Get('SESSION_KEY'))
+        print("port=%d,key=%s" % (ed_ipc.EDPORT,
+                                  profiler.Profile_Get('SESSION_KEY')))
 
     # Check if this is the only instance, if its not exit since
     # any of the opening commands have already been passed to the
