@@ -16,8 +16,8 @@ deduct the requested information.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: pycomp.py 66879 2011-02-10 18:45:12Z CJP $"
-__revision__ = "$Revision: 66879 $"
+__cvsid__ = "$Id: pycomp.py 69384 2011-10-11 20:02:47Z CJP $"
+__revision__ = "$Revision: 69384 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -39,7 +39,7 @@ import completer
 #--------------------------------------------------------------------------#
 
 class Completer(completer.BaseCompleter):
-    """Code completer provider"""
+    """Python Code completion provider"""
     def __init__(self, stc_buffer):
         """Initializes the completer
         @param stc_buffer: buffer that contains code
@@ -84,6 +84,7 @@ class Completer(completer.BaseCompleter):
             if fname:
                 fpath = os.path.dirname(fname)
                 sys.path.insert(0, fpath)
+            snapshot = list(sys.modules.keys())
 
             t1 = time.time()
             cmpl.evalsource(self._buffer.GetText(),
@@ -93,28 +94,28 @@ class Completer(completer.BaseCompleter):
             if fname:
                 sys.path.pop(0)
 
+            # Dump any other modules that got brought in during eval
+            # so that they get properly updated on next pass through.
+            nsnapshot = sys.modules.keys()
+            nimport = list(set(nsnapshot).difference(set(snapshot)))
+            for k in nimport:
+                del sys.modules[k]
+
             if calltip:
                 return cmpl.get_completions(command + u'(', u'', calltip)
             else:
                 # Get Auto-completion List
                 complst = cmpl.get_completions(command)
                 sigs = list()
-                type = completer.TYPE_UNKNOWN
+                tmap = {"function" : completer.TYPE_FUNCTION,
+                        "method" : completer.TYPE_METHOD,
+                        "class" : completer.TYPE_CLASS,
+                        "attribute" : completer.TYPE_ATTRIBUTE,
+                        "property" : completer.TYPE_PROPERTY}
                 for sig in complst:
                     word = sig['word'].rstrip(u'(.')
-                    if sig['type'] == "function":
-                        type = completer.TYPE_FUNCTION
-                    elif sig['type'] == "method":
-                        type = completer.TYPE_METHOD
-                    elif sig['type'] == "class":
-                        type = completer.TYPE_CLASS
-                    elif sig['type'] == "attribute":
-                        type = completer.TYPE_ATTRIBUTE
-                    elif sig['type'] == "property":
-                        type = completer.TYPE_PROPERTY
-                    
-                    sigs.append(completer.Symbol(word, type))
-
+                    tval = tmap.get(sig['type'], completer.TYPE_UNKNOWN)
+                    sigs.append(completer.Symbol(word, tval))
                 sigs.sort(key=lambda x: x.Name.upper())
                 return sigs
 
@@ -198,7 +199,13 @@ class PyCompleter(object):
         """
         scope = self.parser.parse(text.replace('\r\n', '\n'), line)
         src = scope.get_code()
-        dbg("[pycomp] Generated source: %s" % src)
+        # Test
+#        f = open('pycompout.py', 'w')
+#        f.write(src)
+#        f.close()
+        # NOTE: keep commented out in production code can cause slow down
+        #       when running in editra with DEBUG output.
+#        dbg("[pycomp][info] Generated source: %s" % src)
         try: 
             exec src in self.compldict
         except Exception, msg:
@@ -206,7 +213,7 @@ class PyCompleter(object):
         else:
             dbg("[pycomp][info] Successfully executed source code")
 
-        for loc in scope.locals:
+        for loc in [ l[1] for l in scope.locals]:
             try: 
                 exec loc in self.compldict
             except Exception, msg:
@@ -295,19 +302,21 @@ class PyCompleter(object):
             ridx = stmt.rfind('.')
             if len(stmt) > 0 and stmt[-1] == '(':
                 if ctip:
-                    # Use introspect for calltips for now until some
-                    # issues are sorted out with the main parser
+                    # Try introspect.getCallTip since it is generally
+                    # better at getting tips for c modules
                     tip = introspect.getCallTip(_sanitize(stmt), 
                                                 self.compldict)[2]
                     if not isinstance(tip, basestring):
                         tip = u""
+                    if not tip:
+                        # Internal calltip code
+                        result = eval(_sanitize(stmt.rstrip('(')), self.compldict)
+                        doc = max(getattr(result, '__doc__', ''), ' ')
+                        argval = context + _cleanstr(self.get_arguments(result))
+                        tip = '\n'.join([argval, _cleanstr(doc)])
+                        dbg("[pycomp][info] Used internal calltips")
                     return tip
-#                else:
-#                    # Alternate calltip code
-#                    result = eval(_sanitize(stmt[:-1]), self.compldict)
-#                    doc = max(getattr(result, '__doc__', ''), ' ')
-#                    return [{'word' : _cleanstr(self.get_arguments(result)), 
-#                             'info' : _cleanstr(doc)}]
+
             elif ridx == -1:
                 match = stmt
                 compdict = self.compldict
@@ -401,12 +410,36 @@ class Scope(object):
         """
         super(Scope, self).__init__()
 
+        # Attributes
         self.subscopes = list()
         self.docstr = u''
         self.locals = list()
         self.parent = None
         self.name = name
         self.indent = indent
+        self.objid = -1         # Tracks order of declaration
+
+    DocStr = property(lambda self: self.docstr,
+                      lambda self, dstr: setattr(self, 'docstr', dstr))
+    Locals = property(lambda self: self.locals,
+                      lambda self, loc: setattr(self, 'locals', loc))
+    Parent = property(lambda self: self.parent,
+                      lambda self, parent: setattr(self, 'parent', parent))
+
+    def Clone(self, indent=0):
+        """Clone this scope object"""
+        obj = Scope(self.name, indent)
+        obj.DocStr = self.DocStr
+        obj.Locals = list(self.Locals)
+        obj.Parent = self.Parent
+        for scope in self.subscopes:
+            obj.subscopes.append((scope[0], scope[1].Clone(indent + 1)))
+        obj.objid = self.objid
+        return obj
+
+    def NextObjId(self):
+        self.objid += 1
+        return self.objid
 
     def add(self, sub):
         """Push a subscope into this scope
@@ -414,7 +447,7 @@ class Scope(object):
 
         """
         sub.parent = self
-        self.subscopes.append(sub)
+        self.subscopes.append((self.NextObjId(), sub))
         return sub
 
     def doc(self, docstr):
@@ -422,18 +455,15 @@ class Scope(object):
         @param docstr: Docstring to format and set
 
         """
-        dstr = docstr.replace('\n', ' ')
-        dstr = dstr.replace('\t', ' ')
-        dstr = dstr.replace('  ', ' ')
-
+        dstr = docstr
         if len(dstr):
             while len(dstr) and dstr[0] in '"\' ':
                 dstr = dstr[1:]
 
             while len(dstr) and dstr[-1] in '"\' ':
                 dstr = dstr[:-1]
-
-        self.docstr = dstr
+        dstr = '\n'.join([d.lstrip() for d in dstr.split('\n')])
+        self.docstr = dstr.rstrip()
 
     def local(self, loc):
         """Add an object to the scopes locals
@@ -441,15 +471,7 @@ class Scope(object):
 
         """
         self._checkexisting(loc)
-        self.locals.append(loc)
-
-    def copy_decl(self, indent=0):
-        """Copy a scope's declaration only, at the specified indent level 
-        - not local variables
-        @keyword indent: indent level of scope declaration
-
-        """
-        return Scope(self.name, indent)
+        self.locals.append((self.NextObjId(), loc))
 
     def _checkexisting(self, test):
         """Convenience function... keep out duplicates
@@ -457,10 +479,10 @@ class Scope(object):
                      variable in the scopes locals
 
         """
-        if test.find('=') > -1:
+        if '=' in test:
             var = test.split('=')[0].strip()
             for loc in self.locals:
-                if loc.find('=') > -1 and var == loc.split('=')[0].strip():
+                if '=' in loc[1] and var == loc[1].split('=')[0].strip():
                     self.locals.remove(loc)
 
     def get_code(self):
@@ -471,8 +493,8 @@ class Scope(object):
         cstr = '"""' + self.docstr + '"""\n'
         nonimport = list()
         for loc in self.locals:
-            if loc.startswith('import') or loc.startswith('from'):
-                cstr += ("try:\n    %s\nexcept ImportError:\n    pass\n" % loc)
+            if loc[1].startswith('import') or loc[1].startswith('from'):
+                cstr += ("try:\n    %s\nexcept ImportError:\n    pass\n" % loc[1])
             else:
                 nonimport.append(loc)
 
@@ -480,13 +502,13 @@ class Scope(object):
         # hopefully this name is unique enough...
         cstr += 'class _PyCmplNoType:\n    def __getattr__(self,name):\n        return None\n'
 
-        # Get all subscopes (classes, functions, methods)
-        for sub in self.subscopes:
-            cstr += sub.get_code()
-
-        # Get remaining local variables
-        for loc in nonimport:
-            cstr += loc + '\n'
+        decls = self.subscopes + nonimport
+        decls.sort(key=lambda x: x[0])
+        for decl in [d[1] for d in decls]:
+            if isinstance(decl, Scope):
+                cstr += decl.get_code()
+            else:
+                cstr += decl + '\n'
 
         return cstr
 
@@ -528,16 +550,16 @@ class Class(Scope):
         super(Class, self).__init__(name, indent)
         self.supers = supers
 
-    def copy_decl(self, indent=0):
-        """Create a copy of the class object with a scope at the
-        given level of indentation.
-        @keyword indent: scope of indentation
-
-        """
-        cls = Class(self.name, self.supers, indent)
+    def Clone(self, indent=0):
+        """Create a clone of this object"""
+        obj = Class(self.name, self.supers, indent)
+        obj.DocStr = self.DocStr
+        obj.Locals = list(self.Locals)
+        obj.Parent = self.Parent
+        obj.objid = self.objid
         for scope in self.subscopes:
-            cls.add(scope.copy_decl(indent + 1))
-        return cls
+            obj.subscopes.append((scope[0], scope[1].Clone(indent + 1)))
+        return obj
 
     def get_code(self):
         """Get the code string representation of the Class object
@@ -551,13 +573,28 @@ class Class(Scope):
         cstr += ':\n'
         if len(self.docstr) > 0:
             cstr += self.childindent() + '"""' + self.docstr + '"""\n'
-        if len(self.subscopes) > 0:
-            for sub in self.subscopes:
-                cstr += sub.get_code()
-        else:
+        need_pass = True
+        decls = self.locals + self.subscopes
+        decls.sort(key=lambda x: x[0])
+        for decl in [d[1] for d in decls]:
+            need_pass = False
+            if isinstance(decl, Scope):
+                cstr += decl.get_code()
+            else:
+                cstr += ('%s%s\n' % (self.childindent(), decl))
+
+        if need_pass:
             cstr += '%spass\n' % self.childindent()
         return cstr
 
+    def local(self, loc):
+        """Add an object to the scopes locals
+        @param loc: local object to add to locals
+
+        """
+        if loc and '.' not in loc:
+            super(Class, self).local(loc)
+
 class Function(Scope):
     """Create a function object for representing a python function
     definition in the parser.
@@ -573,13 +610,20 @@ class Function(Scope):
         super(Function, self).__init__(name, indent)
         self.params = params
 
-    def copy_decl(self, indent=0):
+    def Clone(self, indent=0):
         """Create a copy of the functions declaration at the given
         scope of indentation.
         @keyword indent: indentation level of the declaration
 
         """
-        return Function(self.name, self.params, indent)
+        obj = Function(self.name, self.params, indent)
+        obj.DocStr = self.DocStr
+        obj.Locals = list(self.Locals)
+        obj.Parent = self.Parent
+        for scope in self.subscopes:
+            obj.subscopes.append((scope[0], scope[1].Clone(indent + 1)))
+        obj.objid = self.objid
+        return obj
 
     def get_code(self):
         """Get code string representation of the function object
@@ -617,6 +661,8 @@ class PyParser(object):
         name = list()
         if pre is None:
             tokentype, token = self.next()[:2]
+            if token == '(':
+                self._parenparse()
             if tokentype != NAME and token != '*':
                 return ('', token)
         else:
@@ -625,6 +671,11 @@ class PyParser(object):
         name.append(token)
         while True:
             tokentype, token = self.next()[:2]
+            if token == '(':
+                # Handle 'self.foo(a,b,c)'
+                self._parenparse()
+                break
+
             if token != '.':
                 break
 
@@ -698,16 +749,16 @@ class PyParser(object):
 
         """
         self.scope = self.scope.pop(indent)
-        tokentype, fname, indent = self.next()
+        tokentype, fname, findent = self.next()
         if tokentype != NAME:
             return None
 
-        tokentype, open_paren, indent = self.next()
+        tokentype, open_paren, tindent = self.next()
         if open_paren != '(':
             return None
 
         params = self._parenparse()
-        tokentype, colon, indent = self.next()
+        tokentype, colon, tindent = self.next()
         if colon != ':':
             return None
 
@@ -758,6 +809,8 @@ class PyParser(object):
                    tokenize.NUMBER : '0', 'ord' : '0', 'id' : '0',
                    'abs' : '0', 'sum' : '0', 'pow' : '0', 'len' : '0',
                    'hash' : '0',
+                   # Property
+                   'property' : 'property()',
                    # String
                    tokenize.STRING : '""', 'str' : '""',
                    'repr' : '""', 'chr' : '""', 'unichr' : '""',
@@ -840,11 +893,16 @@ class PyParser(object):
 
         """
         newscope = Scope('result', 0)
-        scp = self.currentscope
-        while scp != None:
+        scopes = list()
+        tscp = self.currentscope.parent
+        while tscp != None:
+            scopes.append(tscp)
+            tscp = tscp.parent
+        scopes.append(self.currentscope)
+
+        for scp in scopes:
             if type(scp) == Function:
                 cut = 0
-
                 # Handle 'self' params
                 if scp.parent != None and type(scp.parent) == Class:
                     cut = 1
@@ -852,27 +910,26 @@ class PyParser(object):
                     ind = params.find('=')
                     if ind != -1:
                         params = params[:ind]
-                    newscope.local('%s = %s' % (scp.params[0], scp.parent.name))
+                    newscope.local('%s = %s' % (_sanitizeParam(scp.params[0]), scp.parent.name))
 
                 for param in scp.params[cut:]:
                     ind = param.find('=')
                     if len(param) == 0:
                         continue
-
                     if ind == -1:
-                        newscope.local('%s = _PyCmplNoType()' % param)
+                        newscope.local('%s = _PyCmplNoType()' % _sanitizeParam(param))
                     else:
-                        newscope.local('%s = %s' % (param[:ind], 
+                        newscope.local('%s = %s' % (_sanitizeParam(param[:ind]), 
                                                     _sanitize(param[ind+1:])))
 
-            for sub in scp.subscopes:
-                newscope.add(sub.copy_decl(0))
-
-            for loc in scp.locals:
-                newscope.local(loc)
-
-            scp = scp.parent
-
+            decls = scp.subscopes + scp.locals
+            decls.sort(key=lambda x: x[0])
+            for decl in [d[1] for d in decls]:
+                if isinstance(decl, Scope):
+                    newscope.add(decl.Clone(0))
+                else:
+                    newscope.local(decl)
+                
         self.currentscope = newscope
         return self.currentscope
 
@@ -891,20 +948,17 @@ class PyParser(object):
             freshscope = True
             while True:
                 tokentype, token, indent = self.next()
-
                 if tokentype == DEDENT or token == "pass":
                     self.scope = self.scope.pop(indent)
                 elif token == 'def':
                     func = self._parsefunction(indent)
                     if func == None:
-                        #print "function: syntax error..."
                         continue
                     freshscope = True
                     self.scope = self.scope.add(func)
                 elif token == 'class':
                     cls = self._parseclass(indent)
                     if cls == None:
-                        #print "class: syntax error..."
                         continue
                     freshscope = True
                     self.scope = self.scope.add(cls)
@@ -920,7 +974,6 @@ class PyParser(object):
                 elif token == 'from':
                     mod, token = self._parsedotname()
                     if not mod or token != "import":
-                        #print "from: syntax error..."
                         continue
                     names = self._parseimportlist()
                     for name, alias in names:
@@ -934,7 +987,7 @@ class PyParser(object):
                     if freshscope:
                         self.scope.doc(token)
                 elif tokentype == NAME:
-                    name, token = self._parsedotname(token) 
+                    name, token = self._parsedotname(token)
                     if token == '=':
                         stmt = self._parseassignment()
                         dbg("[pycomp] parseassignment: %s = %s" % (name, stmt))
@@ -975,3 +1028,16 @@ def _sanitize(cstr):
         elif level == 0:
             val += char
     return val
+
+def _sanitizeParam(param):
+    """Cleanup a value string
+    value = foo
+    to ensure that 'value' is syntactically correct.
+
+    """
+    for i, c in enumerate(param):
+        if c.isalpha() or c == '_':
+            return param[i:]
+    else:
+        return param # something is wrong so let it bomb
+
