@@ -16,11 +16,11 @@ configuration and presentation options.
 The following items are the options that the AdvancedFindReplaceDialog offers
 over the basic FindReplaceDialog.
 
-  * Hide/Show each option or section indvidually (basic dialog only disables them)
+  * Hide/Show each option or section individually (basic dialog only disables them)
   * Multi-Find/Replace event action for Find All / Replace All actions
   * Switch dialog from Find mode to Replace mode or visa-versa once its already
     been created.
-  * Options for specifiying the location to look in.
+  * Options for specifying the location to look in.
   * Regular Expression option
   * Use standard dialog or a floating MiniFrame (default)
 
@@ -35,22 +35,24 @@ eclib.ctrlbox
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: finddlg.py 61489 2009-07-21 15:54:54Z CJP $"
-__revision__ = "$Revision: 61489 $"
+__svnid__ = "$Id: finddlg.py 70025 2011-12-17 18:16:54Z CJP $"
+__revision__ = "$Revision: 70025 $"
 
 __all__ = ["FindBox", "FindEvent", "FindPanel", "FindReplaceDlg",
            "MiniFindReplaceDlg", "AdvFindReplaceDlg",
 
            "AFR_STYLE_FINDDIALOG", "AFR_STYLE_REPLACEDIALOG",
-           "AFR_STYLE_NON_FLOATING",
+           "AFR_STYLE_NON_FLOATING", "AFR_STYLE_NO_MODE_SELECT",
            
            "AFR_UP", "AFR_WHOLEWORD", 
            "AFR_MATCHCASE", "AFR_REGEX", "AFR_RECURSIVE", "AFR_NOLOOKIN",
            "AFR_NOUPDOWN", "AFR_NOWHOLEWORD", "AFR_NOMATCHCASE", "AFR_NOREGEX",
-           "AFR_NOFILTER", "AFR_NOOPTIONS",
+           "AFR_NOFILTER", "AFR_NOOPTIONS", "AFR_NO_COUNT", "AFR_NO_ALL_BTN",
+           "AFR_SIMPLE",
 
            "LOCATION_CURRENT_DOC", "LOCATION_IN_SELECTION",
-           "LOCATION_OPEN_DOCS", "LOCATION_IN_FILES", "LOCATION_MAX",
+           "LOCATION_OPEN_DOCS", "LOCATION_IN_CURRENT_DIR",
+           "LOCATION_IN_FILES", "LOCATION_MAX",
 
            "edEVT_FIND_CLOSE", "EVT_FIND_CLOSE", "edEVT_FIND", "EVT_FIND",
            "edEVT_FIND_NEXT", "EVT_FIND_NEXT", "edEVT_FIND_ALL", "EVT_FIND_ALL",
@@ -71,9 +73,10 @@ import platebtn
 # Globals
 
 # Style Flags
-AFR_STYLE_FINDDIALOG    = 0
-AFR_STYLE_REPLACEDIALOG = 1
-AFR_STYLE_NON_FLOATING  = 2
+AFR_STYLE_FINDDIALOG        = 0 # Start dialog in Find mode
+AFR_STYLE_REPLACEDIALOG     = 1 # Start dialog in Replace mode
+AFR_STYLE_NON_FLOATING      = 2 # Use a standard dialog as the tlw
+AFR_STYLE_NO_MODE_SELECT    = 4 # Hide the mode selector buttons
 
 # FindReplaceData Flags
 AFR_UP          = 1             # Set dialogs Search Up flag
@@ -88,13 +91,20 @@ AFR_NOMATCHCASE = 256           # Hide the Match Case option in the dialog
 AFR_NOREGEX     = 512           # Hide the Regular Expression option
 AFR_NOFILTER    = 1024          # Hide the File Filter option
 AFR_NOOPTIONS   = 2048          # Hide all options in the dialog
+AFR_NO_COUNT    = 4096          # Hide Count Button
+AFR_NO_ALL_BTN  = 8192          # Hide Find All / Replace All buttons
+
+# Convenience Flags
+AFR_SIMPLE = (AFR_NOLOOKIN | AFR_NOOPTIONS | AFR_NOUPDOWN | \
+              AFR_NO_COUNT | AFR_NO_ALL_BTN)
 
 # Search Location Parameters (NOTE: must be kept in sync with Lookin List)
-LOCATION_CURRENT_DOC  = 0
-LOCATION_IN_SELECTION = 1
-LOCATION_OPEN_DOCS    = 2
-LOCATION_IN_FILES     = 3
-LOCATION_MAX          = 3
+LOCATION_CURRENT_DOC    = 0
+LOCATION_IN_SELECTION   = 1
+LOCATION_OPEN_DOCS      = 2
+LOCATION_IN_CURRENT_DIR = 3
+LOCATION_IN_FILES       = 4
+LOCATION_MAX            = LOCATION_IN_CURRENT_DIR
 
 # Control Names
 FindBoxName = "EdFindBox"
@@ -172,7 +182,7 @@ class FindEvent(wx.PyCommandEvent):
         @keyword flags: Find/Replace flags
 
         """
-        wx.PyCommandEvent.__init__(self, etype, eid)
+        super(FindEvent, self).__init__(etype, eid)
 
         # Attributes
         self._flags = flags
@@ -339,6 +349,7 @@ def AdvFindReplaceDlg(parent, fdata, title, style=AFR_STYLE_FINDDIALOG):
         dlg = FindReplaceDlg(parent, fdata, title, style)
     else:
         dlg = MiniFindReplaceDlg(parent, fdata, title, style)
+
     return dlg
 
 #--------------------------------------------------------------------------#
@@ -377,9 +388,8 @@ class FindReplaceDlgBase:
         self.Bind(_EVT_MODE_CHANGE, self._OnModeChange)
         self.Bind(_EVT_DO_CLOSE_DLG, lambda evt: self._SendCloseEvent())
         self.Bind(wx.EVT_MENU, lambda evt: self._SendCloseEvent(), id=wx.ID_CLOSE)
-        self.Bind(wx.EVT_SET_FOCUS,
-                  lambda evt: self._panel.SetFocus() and evt.Skip())
-        self.Bind(wx.EVT_SHOW, lambda evt: self._panel.SetFocus())
+        self.Bind(wx.EVT_SET_FOCUS, self._OnSetFocus)
+        self.Bind(wx.EVT_SHOW, self._OnShow)
 
     def __DoLayout(self):
         """Layout the dialog"""
@@ -395,8 +405,8 @@ class FindReplaceDlgBase:
         data.SetFindString(self._panel._ftxt.GetValue())
         data.SetReplaceString(self._panel._rtxt.GetValue())
         evt = FindEvent(edEVT_FIND_CLOSE, self.GetId())
-        wx.PostEvent(self.GetParent(), evt)
         self.Hide()
+        wx.PostEvent(self.GetParent(), evt)
 
     def _OnModeChange(self, evt):
         """Update the the dialog when the mode changes"""
@@ -406,6 +416,15 @@ class FindReplaceDlgBase:
             title = self._rtitle
         self.SetTitle(title)
 
+    def _OnSetFocus(self, evt):
+        if self and self._panel:
+            self._panel.SetFocus()
+            evt.Skip()
+
+    def _OnShow(self, evt):
+        if self and self._panel:
+            self._panel.SetFocus()
+
     def GetData(self):
         """Get the FindReplaceData used by this dialog"""
         return self._panel.GetData()
@@ -611,8 +630,8 @@ class FindBox(ctrlbox.ControlBox):
         @param fdata: wx.FindReplaceData
 
         """
-        ctrlbox.ControlBox.__init__(self, parent, id, pos, size,
-                                    wx.TAB_TRAVERSAL|wx.NO_BORDER, name)
+        super(FindBox, self).__init__(parent, id, pos, size,
+                                      wx.TAB_TRAVERSAL|wx.NO_BORDER, name)
 
         # Attributes
         self._fpanel = FindPanel(self, fdata, style=style)
@@ -635,6 +654,9 @@ class FindBox(ctrlbox.ControlBox):
         self.SetControlBar(ctrlbar)
         self.SetWindow(self._fpanel)
 
+        if style & AFR_STYLE_NO_MODE_SELECT:
+            self.GetControlBar().Hide()
+
         # Event Handlers
         self.Bind(wx.EVT_BUTTON, self.OnButton)
 
@@ -686,8 +708,8 @@ class FindPanel(wx.Panel):
         @param fdata: wx.FindReplaceData
 
         """
-        wx.Panel.__init__(self, parent, id, pos, size,
-                          wx.TAB_TRAVERSAL|wx.NO_BORDER, name)
+        super(FindPanel, self).__init__(parent, id, pos, size,
+                                        wx.TAB_TRAVERSAL|wx.NO_BORDER, name)
 
         # Attributes
         # TODO: change to editable combo box when wxMac has native widget
@@ -696,7 +718,7 @@ class FindPanel(wx.Panel):
         self._ftxt = wx.TextCtrl(self, value=fdata.GetFindString())
         self._rtxt = wx.TextCtrl(self, value=fdata.GetReplaceString())
         locations = [_("Current Document"), _("Selected Text"),
-                     _("Open Documents")]
+                     _("Open Documents"), _("Current Directory")]
         self._lookin = wx.Choice(self, ID_LOOKIN, choices=locations)
         self._lookin.SetSelection(0)
         self._filterlbl = wx.StaticText(self, label=_("File Filters:"))
@@ -806,7 +828,7 @@ class FindPanel(wx.Panel):
         # File Filters
         self._sizers['filter'] = wx.BoxSizer(wx.VERTICAL)
         self._filters = wx.TextCtrl(self)
-        tt_txt = _("Enter wildcard shell patterns for matching files (*.txt).")
+        tt_txt = _("Wildcard shell patterns for matching files (*.txt *.html).")
         self._filters.SetToolTipString(tt_txt)
 
         # Disable spell checking on mac for this control
@@ -883,11 +905,25 @@ class FindPanel(wx.Panel):
 
         """
         if find:
-            show = (wx.ID_FIND, ID_COUNT, ID_FIND_ALL, 'fspacer')
-            hide = (wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer')
+            show = [wx.ID_FIND, ID_COUNT, ID_FIND_ALL, 'fspacer']
+            hide = [wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer']
         else:
-            show = (wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer')
-            hide = (ID_FIND_ALL, ID_COUNT, 'fspacer')
+            show = [wx.ID_REPLACE, ID_REPLACE_ALL, 'frspacer']
+            hide = [ID_FIND_ALL, ID_COUNT, 'fspacer']
+
+        # Hide extra buttons as per configured preference
+        flags = self._fdata.GetFlags()
+        if flags & AFR_NO_COUNT and ID_COUNT in show:
+            show.remove(ID_COUNT)
+            hide.append(ID_COUNT)
+
+        if flags & AFR_NO_ALL_BTN:
+            if ID_FIND_ALL in show:
+                show.remove(ID_FIND_ALL)
+                hide.append(ID_FIND_ALL)
+            if ID_REPLACE_ALL in show:
+                show.remove(ID_REPLACE_ALL)
+                hide.append(ID_REPLACE_ALL)
 
         for ctrl in show:
             if isinstance(ctrl, basestring):
@@ -1016,9 +1052,12 @@ class FindPanel(wx.Panel):
             else:
                 evt.SetReplaceString(None)
 
+            if stype >= LOCATION_IN_FILES or stype == LOCATION_IN_CURRENT_DIR:
+                evt.SetFileFilters(self.GetFileFilters())
             if stype >= LOCATION_IN_FILES:
+                # For IN_CURRENT_DIR it is up to client to determine directory
+                # based on what current means to the application.
                 evt.SetDirectory(self._paths.get(lookin_idx, u''))
-                evt.SetFileFilters(self._filters.GetValue())
 
             wx.PostEvent(self.GetParent(), evt)
             return True
