@@ -15,8 +15,8 @@ syntax highlighting of all supported filetypes.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_basestc.py 67152 2011-03-08 23:40:02Z CJP $"
-__revision__ = "$Revision: 67152 $"
+__svnid__ = "$Id: ed_basestc.py 69268 2011-10-01 19:50:54Z CJP $"
+__revision__ = "$Revision: 69268 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -121,28 +121,37 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
 
     #---- Public Methods ----#
 
-    def AddBookmark(self, line=-1):
+    # General marker api
+    def AddMarker(self, marker, line=-1):
         """Add a bookmark and return its handle
+        @param marker: ed_marker.Marker instance
         @keyword line: if < 0 bookmark will be added to current line
 
         """
+        assert isinstance(marker, ed_marker.Marker)
         if line < 0:
             line = self.GetCurrentLine()
-        bmark = ed_marker.Bookmark()
-        bmark.Set(self, line)
-        return bmark.Handle
+        marker.Set(self, line)
+        return marker.Handle
 
-    def RemoveBookmark(self, line):
+    def RemoveMarker(self, marker, line):
         """Remove the book mark from the given line
+        @param marker: ed_marker.Marker instance
         @param line: int
 
         """
-        ed_marker.Bookmark().Set(self, line, delete=True)
+        assert isinstance(marker, ed_marker.Marker)
+        marker.Set(self, line, delete=True)
 
-    def RemoveAllBookmarks(self):
-        """Remove all the bookmarks in the buffer"""
-        ed_marker.Bookmark().DeleteAll(self)
- 
+    def RemoveAllMarkers(self, marker):
+        """Remove all the bookmarks in the buffer
+        @param marker: ed_marker.Marker instance
+
+        """
+        assert isinstance(marker, ed_marker.Marker)
+        marker.DeleteAll(self)
+
+    #-- Breakpoint marker api --#
     def DeleteAllBreakpoints(self):
         """Delete all the breakpoints in the buffer"""
         ed_marker.Breakpoint().DeleteAll(self)
@@ -171,17 +180,17 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             handle = mobj.Handle
         return handle
 
-    def SetBreakpoint(self, line=-1):
+    def SetBreakpoint(self, line=-1, disabled=False):
         """Set a breakpoint marker on the given line
+        @keyword line: line number
+        @keyword disabled: bool
         @return: breakpoint handle
 
         """
-        handle = self._SetBreakpoint(ed_marker.Breakpoint(), line)
-        return handle
-
-    def SetBreakpointDisabled(self, line=-1):
-        """Set a disabled breakpoint indicator."""
-        handle = self._SetBreakpoint(ed_marker.BreakpointDisabled(), line)
+        if not disabled:
+            handle = self._SetBreakpoint(ed_marker.Breakpoint(), line)
+        else:
+            handle = self._SetBreakpoint(ed_marker.BreakpointDisabled(), line)
         return handle
 
     def ShowStepMarker(self, line=-1, show=True):
@@ -480,6 +489,20 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         step = ed_marker.BreakpointStep()
         step.Background = clback
         step.RegisterWithStc(self)
+        ed_marker.StackMarker().RegisterWithStc(self)
+
+        # Other markers
+        errmk = ed_marker.ErrorMarker()
+        errsty = self.GetItemByName('error_style')
+        rgb = eclib.HexToRGB(errsty.GetBack()[1:])
+        errmk.Background = wx.Colour(*rgb)
+        rgb = eclib.HexToRGB(errsty.GetFore()[1:])
+        errmk.Foreground = wx.Colour(*rgb)
+        errmk.RegisterWithStc(self)
+        # Lint Marker
+        ed_marker.LintMarker().RegisterWithStc(self)
+        ed_marker.LintMarkerWarning().RegisterWithStc(self)
+        ed_marker.LintMarkerError().RegisterWithStc(self)
 
     def DoZoom(self, mode):
         """Zoom control in or out
@@ -1081,6 +1104,41 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
             self.EndUndoAction()
             self.SetFocus()
 
+    def GetViewWhiteSpace(self):
+        """Get if view whitespace is turned on
+        @return: bool
+
+        """
+        val = super(EditraBaseStc, self).GetViewWhiteSpace()
+        return val != wx.stc.STC_WS_INVISIBLE
+
+    def SetViewWhiteSpace(self, viewws):
+        """Overrides base method to make it a simple bool toggle"""
+        if viewws:
+            val = wx.stc.STC_WS_VISIBLEALWAYS
+        else:
+            val = wx.stc.STC_WS_INVISIBLE
+        super(EditraBaseStc, self).SetViewWhiteSpace(val)
+
+    def GetWrapMode(self):
+        """Get if word wrap is turned on
+        @return: bool
+
+        """
+        val = super(EditraBaseStc, self).GetWrapMode()
+        return val != wx.stc.STC_WRAP_NONE
+
+    def SetWrapMode(self, wrap):
+        """Overrides base method to make it a simple toggle operation
+        @param wrap: bool
+
+        """
+        if wrap:
+            val = wx.stc.STC_WRAP_WORD
+        else:
+            val = wx.stc.STC_WRAP_NONE
+        super(EditraBaseStc, self).SetWrapMode(val)
+
     def ShowCallTip(self, command):
         """Shows call tip for given command
         @param command: command to  look for calltips for
@@ -1152,11 +1210,10 @@ class EditraBaseStc(wx.stc.StyledTextCtrl, ed_style.StyleMgr):
         @postcondition: all style settings are refreshed in the control
 
         """
-        self.Freeze()
-        self.StyleClearAll()
-        self.SetSyntax(self.GetSyntaxParams())
-        self.DefineMarkers()
-        self.Thaw()
+        with eclib.Freezer(self) as _tmp:
+            self.StyleClearAll()
+            self.SetSyntax(self.GetSyntaxParams())
+            self.DefineMarkers()
         self.Refresh()
 
     def UpdateBaseStyles(self):
