@@ -18,14 +18,13 @@ a gradient using system defined colors.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_cmdbar.py 66841 2011-02-04 22:52:41Z CJP $"
-__revision__ = "$Revision: 66841 $"
+__svnid__ = "$Id: ed_cmdbar.py 69245 2011-09-30 17:52:23Z CJP $"
+__revision__ = "$Revision: 69245 $"
 
 #--------------------------------------------------------------------------#
 # Imports
 import os
 import sys
-import glob
 import re
 import wx
 
@@ -37,7 +36,8 @@ import ed_event
 import ed_msg
 import ebmlib
 import eclib
-from profiler import Profile_Get, Profile_Set
+import ed_basewin
+from profiler import Profile_Get
 
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
@@ -69,7 +69,8 @@ ID_REGEX = wx.NewId()
 
 #-----------------------------------------------------------------------------#
 
-class CommandBarBase(eclib.ControlBar):
+class CommandBarBase(ed_basewin.EdBaseCtrlBar,
+                     ebmlib.FactoryMixin):
     """Base class for control bars"""
     def __init__(self, parent):
         super(CommandBarBase, self).__init__(parent,
@@ -96,14 +97,25 @@ class CommandBarBase(eclib.ControlBar):
         self.Bind(wx.EVT_BUTTON, self.OnClose, self.close_b)
         self.Bind(wx.EVT_CONTEXT_MENU, self.OnContext)
         self.Bind(wx.EVT_MENU, self.OnContextMenu)
+        self.Bind(wx.EVT_SHOW, self.OnShowBar)
+
+    @classmethod
+    def GetMetaDefaults(cls):
+        return dict(id=-1, config_key=None)
+
+    #---- Properties ----#
+
+    MainControl = property(lambda self: self.ctrl,
+                           lambda self, ctrl: self.SetControl(ctrl))
+
+    #---- Implementation ----#
 
     def OnClose(self, evt):
         """Handles events from the buttons on the bar
         @param evt: Event that called this handler
 
         """
-        e_id = evt.GetId()
-        if e_id == ID_CLOSE_BUTTON:
+        if evt.Id == ID_CLOSE_BUTTON:
             self.Hide()
         else:
             evt.Skip()
@@ -115,8 +127,9 @@ class CommandBarBase(eclib.ControlBar):
                 # Lazy init the menu
                 self._menu = wx.Menu(_("Customize"))
                 # Ensure the label is disabled (wxMSW Bug)
-                item = self._menu.GetMenuItems()[0]
-                self._menu.Enable(item.GetId(), False)
+                if len(self._menu.MenuItems):
+                    item = self._menu.MenuItems[0]
+                    self._menu.Enable(item.GetId(), False)
 
                 to_menu = list()
                 for child in self.GetChildren():
@@ -129,10 +142,10 @@ class CommandBarBase(eclib.ControlBar):
                         if not item.GetLabel():
                             continue
 
-                        self._menu.Append(item.GetId(),
+                        self._menu.Append(item.Id,
                                           item.GetLabel(),
                                           kind=wx.ITEM_CHECK)
-                        self._menu.Check(item.GetId(), item.IsShown())
+                        self._menu.Check(item.Id, item.IsShown())
 
             self.PopupMenu(self._menu)
         else:
@@ -140,8 +153,7 @@ class CommandBarBase(eclib.ControlBar):
 
     def OnContextMenu(self, evt):
         """Hide and Show controls"""
-        e_id = evt.GetId()
-        ctrl = self.FindWindowById(e_id)
+        ctrl = self.FindWindowById(evt.Id)
         if ctrl is not None:
             self.ShowControl(ctrl.GetName(), not ctrl.IsShown())
             self.Layout()
@@ -153,6 +165,20 @@ class CommandBarBase(eclib.ControlBar):
                 state = self.GetControlStates()
                 cfg[key] = state
 
+    def OnShowBar(self, evt):
+        """Update the session list"""
+        if evt.IsShown():
+            if self and evt.EventObject is self:
+                self.OnBarShown()
+        evt.Skip()
+
+    def OnBarShown(self):
+        """virtual override for subclasses that wish to receive window show
+        event callbacks.
+
+        """
+        pass
+
     def EnableMenu(self, enable=True):
         """Enable the popup customization menu
         @keyword enable: bool
@@ -164,12 +190,12 @@ class CommandBarBase(eclib.ControlBar):
             self._menu = None
 
     def GetConfigKey(self):
-        """Get the key to use for the layout config persistence.
+        """Get the key to use for the layout config persistence. This value
+        is set in the class definitions meta class.
         @return: string
-        @note: override in subclasses
 
         """
-        return None
+        return self.meta.config_key
 
     def GetControlStates(self):
         """Get the map of control name id's to their shown state True/False
@@ -193,14 +219,16 @@ class CommandBarBase(eclib.ControlBar):
 
     def Hide(self):
         """Hides the control and notifies the parent
-        @postcondition: commandbar is hidden
+        @postcondition: CommandBar is hidden
         @todo: don't reference nb directly here
 
         """
         super(CommandBarBase, self).Hide()
         self._parent.SendSizeEvent()
         nb = self._parent.GetNotebook()
-        nb.GetCurrentCtrl().SetFocus()
+        ctrl = nb.GetCurrentCtrl()
+        if ctrl:
+            ctrl.SetFocus()
         return True
 
     def ShowControl(self, ctrl_name, show=True):
@@ -210,16 +238,16 @@ class CommandBarBase(eclib.ControlBar):
 
         """
         sizer = self.GetControlSizer()
-        next = False
+        gonext = False
         for item in sizer.GetChildren():
-            if next:
+            if gonext:
                 if item.IsSpacer():
                     item.Show(show)
                 break
 
             if item.Window and item.Window.GetName() == ctrl_name:
                 item.Show(show)
-                next = True
+                gonext = True
 
     def IsCustomizable(self, ctrl):
         """Is the control of a type that can be customized
@@ -242,13 +270,17 @@ class CommandBarBase(eclib.ControlBar):
     def SetFocus(self):
         """Set the focus to the bar and its main control"""
         super(CommandBarBase, self).SetFocus()
-        if self.ctrl is not None:
-            self.ctrl.SetFocus()
+        if self.MainControl:
+            self.MainControl.SetFocus()
 
 #-----------------------------------------------------------------------------#
 
 class SearchBar(CommandBarBase):
     """Commandbar for searching text in the current buffer."""
+    class meta:
+        id = ed_glob.ID_QUICK_FIND
+        config_key = 'SearchBar'
+
     def __init__(self, parent):
         super(SearchBar, self).__init__(parent)
 
@@ -316,6 +348,7 @@ class SearchBar(CommandBarBase):
         self.SetControlStates(cfg)
 
     def OnDestroy(self, evt):
+        """Cleanup message handlers on destroy"""
         if evt.GetId() == self.GetId():
             ed_msg.Unsubscribe(self.OnThemeChange)
             self._sctrl.RemoveClient(self)
@@ -372,33 +405,29 @@ class SearchBar(CommandBarBase):
         @param msg: Message Object
 
         """
-        next = self.FindWindowById(ID_SEARCH_NEXT)
-        if next:
+        next_btn = self.FindWindowById(ID_SEARCH_NEXT)
+        if next_btn:
             t_bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_DOWN), wx.ART_MENU)
-            next.SetBitmapLabel(t_bmp)
-            next.SetBitmapHover(t_bmp)
-            next.Update()
-            next.Refresh()
+            next_btn.SetBitmapLabel(t_bmp)
+            next_btn.SetBitmapHover(t_bmp)
+            next_btn.Update()
+            next_btn.Refresh()
 
-        pre = self.FindWindowById(ID_SEARCH_PRE)
-        if pre:
+        pre_btn = self.FindWindowById(ID_SEARCH_PRE)
+        if pre_btn:
             t_bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_UP), wx.ART_MENU)
-            pre.SetBitmapLabel(t_bmp)
-            pre.SetBitmapHover(t_bmp)
-            pre.Update()
-            pre.Refresh()
-
-    def GetConfigKey(self):
-        """Get the key to use for the layout config persistence.
-        @return: string
-
-        """
-        return 'SearchBar'
+            pre_btn.SetBitmapLabel(t_bmp)
+            pre_btn.SetBitmapHover(t_bmp)
+            pre_btn.Update()
+            pre_btn.Refresh()
 
 #-----------------------------------------------------------------------------#
 
 class CommandEntryBar(CommandBarBase):
     """Commandbar for editor command entry and execution."""
+    class meta:
+        id = ed_glob.ID_COMMAND
+
     def __init__(self, parent):
         super(CommandEntryBar, self).__init__(parent)
 
@@ -426,6 +455,9 @@ class CommandEntryBar(CommandBarBase):
 
 class GotoLineBar(CommandBarBase):
     """Commandbar for Goto Line function"""
+    class meta:
+        id = ed_glob.ID_GOTO_LINE
+
     def __init__(self, parent):
         super(GotoLineBar, self).__init__(parent)
 
@@ -486,6 +518,7 @@ class CommandExecuter(eclib.CommandEntryBase):
         ed_msg.Subscribe(self._UpdateCwd, ed_msg.EDMSG_FILE_SAVED)
 
     def OnDestroy(self, evt):
+        """Clean up message handlers on destroy"""
         if evt.GetId() == self.GetId():
             ed_msg.Unsubscribe(self._UpdateCwd)
         evt.Skip()
@@ -696,7 +729,7 @@ class CommandExecuter(eclib.CommandEntryBase):
 
         frame = self.GetTopLevelParent()
         numpage = frame.nb.GetPageCount()
-        for x in xrange(min(count, numpage)):
+        for x in range(min(count, numpage)):
             cpage = frame.nb.GetPageIndex(frame.nb.GetCurrentPage())
             if (cpage == 0 and cmd == 'N') or \
                (cpage + 1 == numpage and cmd == 'n'):
@@ -884,7 +917,7 @@ class CommandExecuter(eclib.CommandEntryBase):
                 wx.CallAfter(self.UpdateAutoComp)
         else:
             if self._popup.HasSuggestions():
-                    self._AdjustValue(self._popup.GetSelection())
+                self._AdjustValue(self._popup.GetSelection())
             self.ListDir()
         self._AdjustSize()
         evt.Skip()
