@@ -15,8 +15,8 @@ main Ui component of the editor that contains all the other components.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_main.py 69119 2011-09-17 18:49:40Z CJP $"
-__revision__ = "$Revision: 69119 $"
+__svnid__ = "$Id: ed_main.py 69319 2011-10-06 21:32:34Z CJP $"
+__revision__ = "$Revision: 69319 $"
 
 #--------------------------------------------------------------------------#
 # Dependencies
@@ -768,7 +768,7 @@ class MainWindow(wx.Frame, viewmgr.PerspectiveManager):
     def DoSaveSessionAs(self):
         """Prompt the user to save the current session"""
         mgr = ed_session.EdSessionMgr()
-        cses = _PGET('LAST_SESSION')
+        cses = _PGET('LAST_SESSION', default=u"")
         if cses == mgr.DefaultSession:
             cses = u""
         fname = ebmlib.LockCall(self._mlock, wx.GetTextFromUser, 
