@@ -13,8 +13,8 @@ This class implements Editra's main notebook control.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_pages.py 69269 2011-10-01 20:14:43Z CJP $"
-__revision__ = "$Revision: 69269 $"
+__svnid__ = "$Id: ed_pages.py 70032 2011-12-17 19:42:54Z CJP $"
+__revision__ = "$Revision: 70032 $"
 
 #--------------------------------------------------------------------------#
 # Dependencies
@@ -363,8 +363,9 @@ class EdPages(ed_book.EdBaseBook):
             flist = mgr.LoadSession(session)
         except Exception, msg:
             self._ses_load = False
+            errdict = dict(sessionname=session, error=msg)
             return (_("Session Load Error"),
-                    _("Failed to load the session: %s\n\nError: %s") % (session, msg))
+                    _("Failed to load the session: %(sessionname)s\n\nError: %(error)s") % errdict)
 
         if not len(flist):
             self._ses_load = False
@@ -482,20 +483,21 @@ class EdPages(ed_book.EdBaseBook):
         @param evt: aui.EVT_AUINOTEBOOK_TAB_MIDDLE_DOWN
 
         """
-        self.mdown = evt.GetSelection()
-        self.SetSelection(self.mdown)
-        self.LOG("[ed_pages][evt] OnMClickDown: %d" % self.mdown)
+        self.mdown = evt.Page
+        idx = self.GetPageIndex(self.mdown)
+        self.SetSelection(idx)
+        self.LOG("[ed_pages][evt] OnMClickDown: %d" % idx)
 
     def OnMClickUp(self, evt):
         """Handle tab middle click event
         @param evt: aui.EVT_AUINOTEBOOK_TAB_MIDDLE_UP
 
         """
-        sel = evt.GetSelection()
+        sel = self.GetPageIndex(evt.Page)
         self.LOG("[ed_pages][evt] OnMClickUp: %d" % sel)
-        if sel == self.mdown:
+        if evt.Page is self.mdown:
             self.ClosePage()
-        self.mdown = -1
+        self.mdown = None
 
     def OnNavigateToPos(self, evt):
         """Handle buffer position history navigation events"""
