@@ -15,8 +15,8 @@ text documents and files.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_search.py 69067 2011-09-11 23:28:42Z CJP $"
-__revision__ = "$Revision: 69067 $"
+__svnid__ = "$Id: ed_search.py 70025 2011-12-17 18:16:54Z CJP $"
+__revision__ = "$Revision: 70025 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -179,7 +179,7 @@ class SearchController(object):
 
         """
         data = msg.GetData()
-        if data.get('mainw', None) == self._parent.GetTopLevelParent():
+        if data.get('mainw', None) == self._parent.TopLevelParent:
 
             if 'findtxt' in data:
                 self.SetQueryString(data.get('findtxt'))
@@ -740,11 +740,12 @@ class SearchController(object):
 
         """
         stc.BeginUndoAction()
+        rstring_utf8 = rstring.encode('utf-8')
         for match in reversed(matches):
             start, end = match.span()
             if isregex:
                 try:
-                    value = match.expand(rstring.encode('utf-8')).decode('utf-8')
+                    value = match.expand(rstring_utf8).decode('utf-8')
                 except re.error, err:
                     msg = _("Error in regular expression expansion."
                             "The replace action cannot be completed.\n\n"
