@@ -14,8 +14,8 @@ Shelf plugin and control implementation
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_shelf.py 67674 2011-05-02 17:35:04Z CJP $"
-__revision__ = "$Revision: 67674 $"
+__svnid__ = "$Id: ed_shelf.py 69269 2011-10-01 20:14:43Z CJP $"
+__revision__ = "$Revision: 69269 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
@@ -31,6 +31,7 @@ import plugin
 import iface
 from extern import aui
 import ed_book
+import ebmlib
 
 #--------------------------------------------------------------------------#
 # Globals
@@ -39,24 +40,6 @@ _ = wx.GetTranslation
 
 #-----------------------------------------------------------------------------#
 
-def mainwinonly(func):
-    """Decorator method to add guards for methods that require the parent
-    to be a MainWindow instance
-
-    """
-    def WrapMainWin(*args, **kwargs):
-        self = args[0]
-        if not hasattr(self._parent, 'GetFrameManager'):
-            return
-        else:
-            func(*args, **kwargs)
-
-    WrapMainWin.__name__ = func.__name__
-    WrapMainWin.__doc__ = func.__doc__
-    return WrapMainWin
-
-#-----------------------------------------------------------------------------#
-
 class Shelf(plugin.Plugin):
     """Plugin that creates a notebook for holding the various Shelf items
     implemented by L{ShelfI}.
@@ -82,15 +65,17 @@ class Shelf(plugin.Plugin):
 
         """
         # First check if the parent has an instance already
-        parent = parent
         mgr = parent.GetFrameManager()
         if mgr.GetPane(Shelf.SHELF_NAME).IsOk():
             return
 
-        shelf = EdShelfBook(parent)
-        mgr.AddPane(shelf,
+        # HACK - fixes mouse event issues that result in wrong
+        #        tab indexes being reported in the notebook.
+        wrapper = ShelfWrapper(parent)
+        shelf = wrapper.GetShelf()
+        mgr.AddPane(wrapper,
                     wx.aui.AuiPaneInfo().Name(Shelf.SHELF_NAME).\
-                            Caption("Shelf").Bottom().Layer(0).\
+                            Caption(_("Shelf")).Bottom().Layer(0).\
                             CloseButton(True).MaximizeButton(True).\
                             BestSize(wx.Size(500,250)))
 
@@ -136,6 +121,8 @@ class Shelf(plugin.Plugin):
 #--------------------------------------------------------------------------#
 
 class EdShelfBook(ed_book.EdBaseBook):
+    ID_SHELF_SUBMENU = wx.NewId()
+    ID_CLOSE_LIKE_TABS = wx.NewId()
     """Shelf notebook control"""
     def __init__(self, parent):
         style = aui.AUI_NB_BOTTOM | \
@@ -147,14 +134,20 @@ class EdShelfBook(ed_book.EdBaseBook):
         super(EdShelfBook, self).__init__(parent, style=style)
 
         # Attributes
-        self._parent = parent
         self._open = dict()
         self._name2idx = dict() # For settings maintenance
-
-        # Setup
-        self.SetSashDClickUnsplit(True)
+        self._menu = ebmlib.ContextMenuManager()
+        self._mcback = None
+        self._mw = None
 
         # Event Handlers
+        # HACK: binding to self works differently than parent catching it
+        #       binding here causes incorrect mouse coords to be sent to
+        #       event handler during click events!
+#        self.Bind(aui.EVT_AUINOTEBOOK_TAB_RIGHT_UP, self.OnTabRightUp)
+        self.Bind(aui.EVT_AUINOTEBOOK_BG_RIGHT_UP, self.OnBgRightUp)
+        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CLOSE, self.OnItemClose)
+        self.Bind(wx.EVT_MENU, self.OnTabMenu)
         self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy, self)
 
         # Message handlers
@@ -163,10 +156,49 @@ class EdShelfBook(ed_book.EdBaseBook):
 
     def OnDestroy(self, evt):
         if evt.GetId() == self.GetId():
+            self._menu.Clear()
             ed_msg.Unsubscribe(self.OnUpdateTabs)
         evt.Skip()
 
+    def OnTabRightUp(self, evt):
+        """Tab right click handler"""
+        self._menu.Clear()
+        if self.MenuCallback:
+            sel = self.GetPageIndex(evt.Page)
+            s_lbl = self.GetPageText(sel)
+            s_lbl = s_lbl.rsplit('-')[0].strip()
+            tab_menu = ed_menu.EdMenu()
+            tab_menu.Append(EdShelfBook.ID_CLOSE_LIKE_TABS,
+                            _("Close All '%s'") % s_lbl)
+            self._menu.AddHandler(EdShelfBook.ID_CLOSE_LIKE_TABS,
+                                  lambda tab, evt: self.CloseAll(s_lbl))
+            tab_menu.AppendSeparator()
+            shelf_menu = self.MenuCallback()
+            tab_menu.AppendMenu(EdShelfBook.ID_SHELF_SUBMENU, 
+                                _("Open"), shelf_menu)
+            self._menu.Menu = tab_menu
+            self.PopupMenu(self._menu.Menu)
+
+    def OnBgRightUp(self, evt):
+        """Show context menu"""
+        self._menu.Clear()
+        if self.MenuCallback:
+            self._menu.Menu = self.MenuCallback()
+            self.PopupMenu(self._menu.Menu)
+
+    def OnTabMenu(self, evt):
+        """Handle tab menu events"""
+        handler = self._menu.GetHandler(evt.Id)
+        if handler is not None:
+            handler(self.GetCurrentPage(), evt)
+        else:
+            evt.Skip()
+
+    MainWindow = property(lambda self: self._mw,
+                          lambda self, mw: setattr(self, '_mw', mw))
     BitmapCallbacks = property(lambda self: self._name2idx)
+    MenuCallback = property(lambda self: self._mcback,
+                            lambda self, funct: setattr(self, '_mcback', funct))
 
     def AddItem(self, item, name, bmp=wx.NullBitmap):
         """Add an item to the shelf's notebook. This is useful for interacting
@@ -183,18 +215,47 @@ class EdShelfBook(ed_book.EdBaseBook):
                      select=True)
 
         # Set the tab icon
-        if Profile_Get('TABICONS', default=True):
-            self.SetPageBitmap(self.GetPageCount()-1, bmp)
+        self.SetPageBitmap(self.GetPageCount()-1, bmp)
         self._open[name] = self._open.get(name, 0) + 1
 
-    @mainwinonly
+    def OnItemClose(self, evt):
+        """Callback for when pages are closed"""
+#        name = self.GetPageText(evt.GetSelection())
+#        name = name.rsplit('-')[0].strip()
+#        self._open[name] = self._open.get(name, 1) - 1
+        evt.Skip()
+
+    def CloseAll(self, name):
+        """Close all of 'name' shelf items
+        @param name: shelf item name (string)
+
+        """
+        idxs = self.GetAllIndexes(name)
+        idxs.reverse()
+        for idx in idxs:
+            self.DeletePage(idx)
+
+    def GetAllIndexes(self, name):
+        """Get all the indexes of pages of the item identified
+        by 'name'.
+        @param name: shelf item name (string)
+
+        """
+        idxs = list()
+        for pg in range(self.GetPageCount()):
+            lbl = self.GetPageText(pg)
+            lbl = PGNUM_PAT.sub(u"", lbl)
+            if lbl == name:
+                idxs.append(pg)
+        return idxs
+
     def EnsureShelfVisible(self):
         """Make sure the Shelf is visible
         @precondition: Shelf.Init has been called
         @postcondition: Shelf is shown
 
         """
-        mgr = self._parent.GetFrameManager()
+        mgr = self.MainWindow.GetFrameManager()
         pane = mgr.GetPane(Shelf.SHELF_NAME)
         if not pane.IsShown():
             pane.Show()
@@ -207,7 +268,7 @@ class EdShelfBook(ed_book.EdBaseBook):
 
         """
         count = 0
-        for page in xrange(self.GetPageCount()):
+        for page in range(self.GetPageCount()):
             if self.GetPageText(page).startswith(item_name):
                 count = count + 1
         return count
@@ -217,7 +278,7 @@ class EdShelfBook(ed_book.EdBaseBook):
         @return: ed_main.MainWindow
 
         """
-        return self._parent
+        return self.MainWindow
 
     def GetOpen(self):
         """Get the list of open shelf items
@@ -226,13 +287,12 @@ class EdShelfBook(ed_book.EdBaseBook):
         """
         return self._open
 
-    @mainwinonly
     def Hide(self):
         """Hide the shelf
         @postcondition: Shelf is hidden by aui manager
 
         """
-        mgr = self._parent.GetFrameManager()
+        mgr = self.MainWindow.GetFrameManager()
         pane = mgr.GetPane(Shelf.SHELF_NAME)
         if pane.IsOk():
             pane.Hide()
@@ -244,18 +304,17 @@ class EdShelfBook(ed_book.EdBaseBook):
         @param item_name: name of Item to look for
 
         """
-        for page in xrange(self.GetPageCount()):
+        for page in range(self.GetPageCount()):
             if self.GetPageText(page).startswith(item_name):
                 return True
         return False
 
-    @mainwinonly
     def IsShown(self):
         """Is the shelf visible?
         @return: bool
 
         """
-        mgr = self._parent.GetFrameManager()
+        mgr = self.MainWindow.GetFrameManager()
         pane = mgr.GetPane(Shelf.SHELF_NAME)
         if pane.IsOk():
             return pane.IsShown()
@@ -294,6 +353,9 @@ class EdShelfDelegate(object):
         self._shelf = shelf
         self._pin = pobject
 
+        # Setup
+        self._shelf.MenuCallback = getattr(self, 'GetShelfObjectMenu')
+
     @property
     def observers(self):
         return self._pin.observers
@@ -350,7 +412,7 @@ class EdShelfDelegate(object):
         """
         rval = list()
         if self._shelf is not None:
-            for page in xrange(self._shelf.GetPageCount()):
+            for page in range(self._shelf.GetPageCount()):
                 rval.append(re.sub(PGNUM_PAT, u'', 
                             self._shelf.GetPageText(page), 1))
         return rval
@@ -380,16 +442,13 @@ class EdShelfDelegate(object):
             except Exception, msg:
                 self._log("[shelf][err] Failed LoadPerspective: %s" % msg)
 
-    def GetMenu(self):
-        """Return the menu of this object
-        @return: ed_menu.EdMenu()
+    def GetShelfObjectMenu(self):
+        """Get the minimal menu that lists all Shelf objects
+        without the 'Show Shelf' item.
+        @return: ed_menu.EdMenu
 
         """
         menu = ed_menu.EdMenu()
-        menu.Append(ed_glob.ID_SHOW_SHELF, _("Show Shelf") + \
-                    ed_menu.EdMenuBar.keybinder.GetBinding(ed_glob.ID_SHOW_SHELF), 
-                    _("Show the Shelf"))
-        menu.AppendSeparator()
         menu_items = list()
         open_items = self._shelf.GetOpen()
         for observer in self.observers:
@@ -409,8 +468,22 @@ class EdShelfDelegate(object):
             shortcut = u""
             if combo < 10:
                 shortcut = u"\tCtrl+Alt+" + unicode(combo)
-            item[1].SetText(item[1].GetText() + shortcut)
-            menu.AppendItem(item[1])
+            nitem = menu.Append(item[1].Id, item[1].GetText() + shortcut)
+            if item[1].Bitmap.IsOk():
+                nitem.SetBitmap(item[1].Bitmap)
+            item[1].Destroy()
+        return menu
+
+    def GetMenu(self):
+        """Return the menu of this object
+        @return: ed_menu.EdMenu()
+
+        """
+        menu = self.GetShelfObjectMenu()
+        menu.Insert(0, wx.ID_SEPARATOR)
+        menu.Insert(0, ed_glob.ID_SHOW_SHELF, _("Show Shelf") + \
+                    ed_menu.EdMenuBar.keybinder.GetBinding(ed_glob.ID_SHOW_SHELF), 
+                    _("Show the Shelf"))
         return menu
 
     def GetOwnerWindow(self):
@@ -446,7 +519,7 @@ class EdShelfDelegate(object):
                 mgr = parent.GetFrameManager()
                 pane = mgr.GetPane(Shelf.SHELF_NAME)
                 if pane is not None:
-                    page = pane.window.GetCurrentPage()
+                    page = pane.window.Book.GetCurrentPage()
                     if hasattr(page, 'SetFocus'):
                         page.SetFocus()
         else:
@@ -499,7 +572,7 @@ class EdShelfDelegate(object):
         @return: reference to the selected page or None if no instance is
 
         """
-        for page in xrange(self._shelf.GetPageCount()):
+        for page in range(self._shelf.GetPageCount()):
             if self._shelf.GetPageText(page).startswith(item_name):
                 self._shelf.SetSelection(page)
                 return self._shelf.GetPage(page)
@@ -567,3 +640,27 @@ class EdShelfDelegate(object):
             evt.Enable(False)
         else:
             evt.Enable(True)
+
+#-----------------------------------------------------------------------------#
+
+class ShelfWrapper(wx.Panel):
+    def __init__(self, parent):
+        super(ShelfWrapper, self).__init__(parent)
+
+        self._shelf = EdShelfBook(self)
+        self._shelf.MainWindow = parent
+
+        # Setup
+        sizer = wx.BoxSizer()
+        sizer.Add(self._shelf, 1, wx.EXPAND)
+        self.SetSizer(sizer)
+
+        self.Bind(aui.EVT_AUINOTEBOOK_TAB_RIGHT_UP, self.OnTabRightUp)
+
+    Book = property(lambda self: self._shelf)
+
+    def GetShelf(self):
+        return self._shelf
+
+    def OnTabRightUp(self, evt):
+        self._shelf.OnTabRightUp(evt)
