@@ -15,8 +15,8 @@ messages from ed_msg to display progress of different actions.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_statbar.py 66181 2010-11-17 14:53:51Z CJP $"
-__revision__ = "$Revision: 66181 $"
+__svnid__ = "$Id: ed_statbar.py 69063 2011-09-11 18:19:32Z CJP $"
+__revision__ = "$Revision: 69063 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -27,6 +27,7 @@ import wx.stc
 import ed_glob
 import util
 import ed_msg
+import ed_menu
 from syntax.synglob import GetDescriptionFromId
 from eclib import ProgressStatusBar, EncodingDialog
 from extern.decorlib import anythread
@@ -51,6 +52,7 @@ class EdStatBar(ProgressStatusBar):
         self._widths = list()
         self._cleanup_timer = wx.Timer(self, EdStatBar.ID_CLEANUP_TIMER)
         self._eolmenu = wx.Menu()
+        self._lexmenu = None
         self._log = wx.GetApp().GetLog()
 
         # Setup
@@ -67,6 +69,7 @@ class EdStatBar(ProgressStatusBar):
                              kind=wx.ITEM_CHECK)
 
         # Event Handlers
+        self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy, self)
         self.Bind(wx.EVT_LEFT_DCLICK, self.OnLeftDClick)
         self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
         self.Bind(wx.EVT_TIMER, self.OnExpireMessage,
@@ -81,12 +84,17 @@ class EdStatBar(ProgressStatusBar):
         ed_msg.Subscribe(self.OnUpdateDoc, ed_msg.EDMSG_FILE_OPENED)
         ed_msg.Subscribe(self.OnUpdateDoc, ed_msg.EDMSG_UI_STC_LEXER)
 
-    def __del__(self):
+    def OnDestroy(self, evt):
         """Unsubscribe from messages"""
-        ed_msg.Unsubscribe(self.OnProgress)
-        ed_msg.Unsubscribe(self.OnUpdateText)
-        ed_msg.Unsubscribe(self.OnUpdateDoc)
-        super(EdStatBar, self).__del__()
+        if self._lexmenu:
+            self._lexmenu.Destroy()
+        if self._eolmenu:
+            self._eolmenu.Destroy()
+        if evt.GetId() == self.GetId():
+            ed_msg.Unsubscribe(self.OnProgress)
+            ed_msg.Unsubscribe(self.OnUpdateText)
+            ed_msg.Unsubscribe(self.OnUpdateDoc)
+        evt.Skip()
 
     def __SetStatusText(self, txt, field):
         """Safe method to use for setting status text with CallAfter.
@@ -142,7 +150,7 @@ class EdStatBar(ProgressStatusBar):
 
     def GetMainWindow(self):
         """Method required for L{ed_msg.mwcontext}"""
-        return self.GetParent()
+        return self.TopLevelParent
 
     def OnExpireMessage(self, evt):
         """Handle Expiring the status message when the oneshot timer
@@ -203,6 +211,14 @@ class EdStatBar(ProgressStatusBar):
             #       check to ensure reference is still valid.
             if dlg:
                 dlg.Destroy()
+        elif self.GetFieldRect(ed_glob.SB_LEXER).Contains(pt):
+            # Change Lexer popup menu
+            if self._lexmenu:
+                self._lexmenu.Destroy()
+            self._lexmenu = wx.Menu()
+            ed_menu.EdMenuBar.PopulateLexerMenu(self._lexmenu)
+            rect = self.GetFieldRect(ed_glob.SB_LEXER)
+            self.PopupMenu(self._lexmenu, (rect.x, rect.y))
         else:
             evt.Skip()
 
