@@ -18,8 +18,8 @@ specific options such as commenting code.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_stc.py 69065 2011-09-11 19:18:25Z CJP $"
-__revision__ = "$Revision: 69065 $"
+__svnid__ = "$Id: ed_stc.py 70024 2011-12-17 16:37:13Z CJP $"
+__revision__ = "$Revision: 70024 $"
 
 #-------------------------------------------------------------------------#
 # Imports
@@ -732,17 +732,15 @@ class EditraStc(ed_basestc.EditraBaseStc):
     def DoAutoComplete(self):
         """Atempt to perform an autocompletion event."""
         self.HidePopups()
-        if self._config['autocomp']:
-            command = self.GetCommandStr()
-            self.ShowAutoCompOpt(command)
+        command = self.GetCommandStr()
+        self.ShowAutoCompOpt(command)
 
     def DoCallTip(self):
         """Attempt to show a calltip for the current cursor position"""
         self.HidePopups()
-        if self._config['autocomp']:
-            command = self.GetCommandStr()
-            # TODO: GetCommandStr seems to be inadquate under some cases
-            self.ShowCallTip(command)
+        command = self.GetCommandStr()
+        # TODO: GetCommandStr seems to be inadequate under some cases
+        self.ShowCallTip(command)
 
     def OnKeyUp(self, evt):
         """Update status bar of window
