@@ -20,8 +20,8 @@ U{http://editra.org/editra_style_sheets}.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_style.py 68615 2011-08-09 19:01:24Z CJP $"
-__revision__ = "$Revision: 68615 $"
+__svnid__ = "$Id: ed_style.py 69752 2011-11-13 23:15:40Z CJP $"
+__revision__ = "$Revision: 69752 $"
 
 #--------------------------------------------------------------------------#
 # Imports
@@ -34,6 +34,7 @@ import ed_glob
 import util
 from profiler import Profile_Get, Profile_Set
 import eclib
+import ebmlib
 
 # Globals
 STY_ATTRIBUTES     = (u"face", u"fore", u"back", u"size", u"modifiers")
@@ -581,16 +582,14 @@ class StyleMgr(object):
         """
         if sheet_name:
             style = sheet_name
-            if sheet_name.split(u'.')[-1] != u"ess":
-                style += u".ess"
-        elif Profile_Get('SYNTHEME', 'str').split(u'.')[-1] != u"ess":
-            style = (Profile_Get('SYNTHEME', 'str') + u".ess").lower()
         else:
-            style = Profile_Get('SYNTHEME', 'str').lower()
+            style = Profile_Get('SYNTHEME', 'str')
+        style = ebmlib.AddFileExtension(style, u'.ess').lower()
 
         # Get Correct Filename if it exists
-        for sheet in util.GetResourceFiles(u'styles', False, True, title=False):
-            if sheet.lower() == style.lower():
+        for sheet in util.GetResourceFiles(u'styles', trim=False, 
+                                           get_all=True, title=False):
+            if sheet.lower() == style:
                 style = sheet
                 break
 
@@ -994,7 +993,7 @@ class StyleMgr(object):
         if sum(sback.Get()) < 384:
             self.SetSelForeground(True, wx.WHITE)
         else:
-            self.SetSelForeground(False, wx.BLACK)
+            self.SetSelForeground(True, wx.BLACK)
         self.SetSelBackground(True, sback)
 
         # Causes issues with selecting text when view whitespace is on
