@@ -16,8 +16,8 @@ and setting of the program by setting values in the Profile.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: prefdlg.py 69268 2011-10-01 19:50:54Z CJP $"
-__revision__ = "$Revision: 69268 $"
+__svnid__ = "$Id: prefdlg.py 69752 2011-11-13 23:15:40Z CJP $"
+__revision__ = "$Revision: 69752 $"
 
 #----------------------------------------------------------------------------#
 # Dependencies
@@ -1080,7 +1080,7 @@ class DocSyntaxPanel(wx.Panel):
         # Syntax Settings
         syn_cb = wx.CheckBox(self, ed_glob.ID_SYNTAX, _("Syntax Highlighting"))
         syn_cb.SetValue(Profile_Get('SYNTAX'))
-        ss_lst = util.GetResourceFiles(u'styles', get_all=True)
+        ss_lst = util.GetResourceFiles(u'styles', get_all=True, title=False)
         ss_lst = [sheet for sheet in ss_lst if not sheet.startswith('.')]
         syntheme = ExChoice(self, ed_glob.ID_PREF_SYNTHEME,
                             choices=sorted(ss_lst),
